#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Tests for casa_envoy Auditor
"""
from pathlib import Path
from unittest.mock import patch

from casa_envoy.auditor import (
    AuditDirectories,
    AuditFiles,
    get_fields_for,
    get_json_content,
    get_value_for,
    get_xml_content,
)
from casa_envoy.schema import AbstractTextFile

test_ppr = AbstractTextFile(filename="test/input_files/PPR.xml", content=Path("test/input_files/PPR.xml").read_text())
test_ppr_numerical = AbstractTextFile(
    filename="test/input_files/PPR-numerical-path.xml",
    content=Path("test/input_files/PPR-numerical-path.xml").read_text(),
)
test_cal_metadata = AbstractTextFile(
    filename="test/input_files/test.json", content=Path("test/input_files/test.json").read_text()
)

test_restore_metadata = AbstractTextFile(
    "test/input_files/restore.json", content=Path("test/input_files/restore.json").read_text()
)
test_restore_ppr = AbstractTextFile(
    "test/input_files/restore-PPR.xml", content=Path("test/input_files/restore-PPR.xml").read_text()
)

test_img_ppr = AbstractTextFile(
    filename="test/input_files/cmsimage-PPR.xml",
    content=Path("test/input_files/cmsimage-PPR.xml").read_text(),
)
test_img_metadata = AbstractTextFile(
    filename="test/input_files/image-metadata.json",
    content=Path("test/input_files/image-metadata.json").read_text(),
)


def test_get_fields_for():
    """
    Verify required fields are present in provided files

    :return:
    """
    product_type = "standard-cal"
    fields = [
        "fileSetIds",
        "workflowName",
        "systemId",
        "creationTime",
        "productLocator",
        "destinationDirectory",
    ]
    result = get_fields_for(product_type=product_type, filename=test_cal_metadata.filename)
    assert result == fields
    fields2 = ["RootDirectory", "RelativePath", "SdmIdentifier"]
    result2 = get_fields_for(product_type=product_type, filename=test_ppr.filename)
    assert result2 == fields2

    product_type2 = "standard-img"
    img_fields = [
        "fileSetIds",
        "workflowName",
        "systemId",
        "creationTime",
        "productLocator",
        "destinationDirectory",
        "calibrationSourceDirectory",
        "cmsName",
    ]
    result = get_fields_for(product_type=product_type2, filename=test_img_metadata.filename)
    assert result == img_fields


def test_get_xml_content():
    """
    Verify retrieval of xml content

    :return:
    """
    content = get_xml_content(test_ppr)
    # is content accessible to the xml parser?
    assert content.find("RelativePath").string == "tmpiox5trbp"


def test_get_json_content():
    """
    Verify retrieval of json content

    :return:
    """
    content = get_json_content(test_cal_metadata)
    # is content accessible as json?
    assert content["productLocator"] == "uid://evla/execblock/9cb0964b-ad6b-40ed-bd87-d08c502503e2"


def test_get_value_for():
    """

    :return:
    """
    test_xml_content = get_xml_content(test_ppr)
    value = get_value_for(test_xml_content, "RelativePath")
    assert value == "tmpiox5trbp"
    value = get_value_for(test_xml_content, "SdmIdentifier")
    assert value == "brain_001.58099.678886747686"

    test_json_content = get_json_content(test_cal_metadata)
    value = get_value_for(test_json_content, "workflowName")
    assert value == "std_calibration"
    value = get_value_for(test_json_content, "productLocator")
    assert value == "uid://evla/execblock/9cb0964b-ad6b-40ed-bd87-d08c502503e2"


class TestAuditFiles:
    """
    Test methods for file auditing
    """

    def test_check_required_fields(self, cal_settings):
        audit_files = AuditFiles(test_ppr, [test_cal_metadata], cal_settings)
        fields = [
            "fileSetIds",
            "workflowName",
            "systemId",
            "creationTime",
            "productLocator",
            "destinationDirectory",
        ]

        result = audit_files.check_required_fields(test_cal_metadata, fields)
        assert result is True

        fieldsF = [
            "fileSetIds",
            "workflowName",
            "systemId",
            "creationTime",
            "productLocator",
            "destinationDirectory",
            "FakeField",
        ]

        resultF = audit_files.check_required_fields(test_cal_metadata, fieldsF)
        assert resultF is False

        fields2 = ["RootDirectory", "RelativePath", "SdmIdentifier"]

        result2 = audit_files.check_required_fields(test_ppr, fields2)
        assert result2 is True

        fields2F = ["RootDirectory", "RelativePath", "SdmIdentifier", "FakeField"]

        result2F = audit_files.check_required_fields(test_ppr, fields2F)
        assert result2F is False

    @patch("builtins.open")
    @patch("os.chdir")
    @patch("shutil.copy")
    def test_correct_for_condor(self, mock_copy, mock_os, mock_open, cal_settings):
        ppr = AuditFiles(test_ppr, [test_cal_metadata], cal_settings).correct_for_condor(ppr=test_ppr)
        assert mock_open.call_count == 1  # would this have written out as expected if not mocked?
        assert ppr.filename == "test/input_files/PPR.xml"

    @patch("builtins.open")
    @patch("os.chdir")
    @patch("shutil.copy")
    def test_correct_for_condor_numerical_path(self, mock_copy, mock_os, mock_open, cal_settings):
        cal_settings_num = cal_settings.copy()
        cal_settings_num["ppr"] = "test/input_files/PPR-numerical-path.xml"
        cal_settings_num["processingDirectory"] = "1234567"
        ppr = AuditFiles(test_ppr_numerical, [test_cal_metadata], cal_settings_num).correct_for_condor(
            ppr=test_ppr_numerical
        )
        correct_content = Path("test/input_files/PPR-numerical-path-correct.xml").read_text()
        assert mock_open.call_count == 1  # would this have written out as expected if not mocked?
        assert ppr.content == correct_content

    def test_audit_cal(self, cal_settings):
        with patch("casa_envoy.auditor.AuditFiles.correct_for_condor") as condor:
            result = AuditFiles(test_ppr, [test_cal_metadata], cal_settings).audit()
            assert result is True

    def test_audit_cal_bad_no_ppr(self, cal_settings):
        with patch("casa_envoy.auditor.AuditFiles.correct_for_condor") as condor:
            result = AuditFiles(None, [test_cal_metadata], cal_settings).audit()
            assert result is False

    def test_audit_cal_bad_wrong_product_type(self, cal_settings):
        with patch("casa_envoy.auditor.AuditFiles.correct_for_condor") as condor:
            cal_settings["product_type"] = "standard-img"
            result2 = AuditFiles(test_ppr, [test_cal_metadata], cal_settings).audit()
            assert result2 is False

    def test_audit_restore_no_ppr(self, restore_settings_sans_ppr):
        with patch("casa_envoy.auditor.AuditFiles.correct_for_condor") as condor:
            result = AuditFiles(None, [test_restore_metadata], restore_settings_sans_ppr).audit()
            assert result is True

    def test_audit_restore_no_ppr_bad_no_piperestorescript(self, restore_settings_sans_ppr):
        del restore_settings_sans_ppr["piperestorescript_path"]
        with patch("casa_envoy.auditor.AuditFiles.correct_for_condor") as condor:
            result2 = AuditFiles(None, [test_restore_metadata], restore_settings_sans_ppr).audit()
            assert result2 is False

    def test_audit_restore_with_ppr_and_piperestorescript(self, restore_settings_sans_ppr):
        with patch("casa_envoy.auditor.AuditFiles.correct_for_condor") as condor:
            result2 = AuditFiles(test_restore_ppr, [test_restore_metadata], restore_settings_sans_ppr).audit()
            assert result2 is True

    def test_audit_restore_with_ppr_no_pipescript(self, restore_settings_sans_ppr):
        del restore_settings_sans_ppr["piperestorescript_path"]
        with patch("casa_envoy.auditor.AuditFiles.correct_for_condor") as condor:
            result2 = AuditFiles(test_restore_ppr, [test_restore_metadata], restore_settings_sans_ppr).audit()
            assert result2 is True


class TestAuditDirectories:
    """
    Test methods for directory auditing
    """

    @patch("glob.glob")
    @patch("os.listdir", return_value=["file1", "file2"])
    @patch("pathlib.Path.is_dir", return_value=True)
    @patch("os.getcwd", return_value="/tmp/workspaces_tmp/tmpiox5trbp")
    def test_audit(self, mock_cwd, mock_isdir, mock_listdir, mock_glob, cal_settings):
        result = AuditDirectories(test_cal_metadata, cal_settings).audit()
        assert result is True
