#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Tests that test the view functionality of our capability REST API.
The logic can be found in `capability/views/capability.py`.
"""
import http
from typing import Optional

import pytest
from pyramid.config import Configurator
from pyramid.httpexceptions import (
    HTTPBadRequest,
    HTTPException,
    HTTPExpectationFailed,
    HTTPNotFound,
    HTTPPreconditionFailed,
)
from pyramid.testing import DummyRequest


def test_view_capability(test_config: Configurator, request_null_capability: DummyRequest):
    """
    Tests the view_capability view to make sure it properly returns the info of a capability

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability import view_capability

    expected_response = {
        "type": "Capability",
        "enabled": True,
        "max_jobs": 2,
        "name": "null",
        "has_image_products": False,
        "requires_qa": None,
        "single_version_only": None,
        "paused": False,
        "state_machine": "None",
        "casa_recipe": None,
        "default_casa_capo_key": None,
    }
    request_null_capability.matchdict["capability_name"] = "null"
    response = view_capability(request_null_capability)
    assert response.status_code == http.HTTPStatus.OK
    assert response.json_body == expected_response


def test_view_capability_not_found(test_config: Configurator, request_null_capability: DummyRequest):
    """
    Tests the view capability view to make sure it properly returns a 404 Not Found exception
    when a capability does not exist

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability import view_capability

    request_null_capability.matchdict["capability_name"] = "does_not_exist"
    response = view_capability(request_null_capability)
    assert response.status_code == http.HTTPStatus.NOT_FOUND
    assert type(response) is HTTPNotFound


def test_view_enabled_capabilities(test_config: Configurator, request_view_enabled_capabilities: DummyRequest):
    """
    Tests the view_capability view to make sure it properly returns the info of a capability

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_view_enabled_capabilities: Dummy Pyramid request object set up with mocked DB access
    to get enabled capabilities
    """
    from capability.views.capability import view_enabled_capabilities

    expected_response = {
        "enabled_capabilities": [
            "null",
            "error",
            "std_restore_imaging",
        ]
    }
    response = view_enabled_capabilities(request_view_enabled_capabilities)
    assert response.status_code == http.HTTPStatus.OK
    assert response.json_body == expected_response


@pytest.mark.skip("Broken due to out of date create_capability endpoint. update endpoint to fix test")
def test_create_capability(test_config: Configurator, request_null_capability: DummyRequest):
    """
    Tests the create capability view to make sure it properly supports creation of capabilities

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability import create_capability

    request_null_capability.json_body = {
        "capability_name": "test_create",
        "max_jobs": 1,
        "has_image_products": False,
        "state_machine_type": "simple",
    }

    # Assert test capability not in list of capabilities (mocked)
    assert not request_null_capability.capability_info.lookup_capability("test")

    response = create_capability(request_null_capability)
    assert response.status_code == http.HTTPStatus.OK
    expected_response = {
        "type": "Capability",
        "enabled": True,
        "max_jobs": 1,
        "name": "test_create",
        "has_image_products": False,
        "requires_qa": None,
        "single_version_only": None,
        "paused": False,
        "state_machine": "simple",
        "casa_recipe": None,
        "default_casa_capo_key": None,
    }
    assert response.json_body == expected_response
    # Assert test capability has been added to list of capabilities (mocked)
    assert request_null_capability.capability_info.lookup_capability("test_create")


def test_create_capability_error(test_config: Configurator, request_null_capability: DummyRequest):
    """
    Tests that the create_capability view correctly responds with exceptions given bad input

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability import create_capability

    request_null_capability.json_body = {"capability_name": "test"}
    response_bad_params = create_capability(request_null_capability)
    assert response_bad_params.status_code == http.HTTPStatus.BAD_REQUEST
    assert type(response_bad_params) is HTTPBadRequest

    request_null_capability.json_body = {"capability_name": "null", "max_jobs": 69, "has_image_products": False}
    response_already_exists = create_capability(request_null_capability)
    assert response_already_exists.status_code == http.HTTPStatus.PRECONDITION_FAILED
    assert type(response_already_exists) is HTTPPreconditionFailed


def test_edit_capability(test_config: Configurator, request_null_capability: DummyRequest):
    """
    Tests that the edit_capability view correctly supports editing of capabilities

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability import edit_capability

    request_null_capability.matchdict["capability_name"] = "null"
    request_null_capability.json_body = {"max_jobs": 3, "has_image_products": False}
    response_all_params = edit_capability(request_null_capability)
    assert response_all_params.status_code == http.HTTPStatus.OK

    request_null_capability.json_body = {"max_jobs": 3}
    response_jobs_only = edit_capability(request_null_capability)
    assert response_jobs_only.status_code == http.HTTPStatus.OK


@pytest.mark.parametrize(
    ["capability_name", "max_jobs", "response_type", "is_raised"],
    [
        ("not_found", 3, HTTPPreconditionFailed, False),
        ("null", None, HTTPBadRequest, False),
        ("error", 3, HTTPExpectationFailed, True),
    ],
)
def test_edit_capability_error(
    test_config: Configurator,
    request_null_capability: DummyRequest,
    capability_name: str,
    max_jobs: int | None,
    response_type: HTTPException,
    is_raised: bool,
):
    """
    Tests that the edit_capability view correctly responds with exceptions

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability import edit_capability

    request_null_capability.matchdict["capability_name"] = capability_name
    if max_jobs is not None:
        request_null_capability.json_body = {"max_jobs": max_jobs}
    else:
        request_null_capability.json_body = {}
    if is_raised:
        with pytest.raises(response_type):
            response = edit_capability(request_null_capability)
            assert response.status_code == response_type.code
            assert type(response) is response_type
    else:
        response = edit_capability(request_null_capability)
        assert response.status_code == response_type.code
        assert type(response) is response_type


def test_enable_capability(test_config: Configurator, request_null_capability: DummyRequest):
    """
    Tests that capabilities can be properly enabled

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability import enable_capability

    request_null_capability.matchdict["capability_name"] = "null"
    response = enable_capability(request_null_capability)
    assert response.status_code == http.HTTPStatus.OK


@pytest.mark.parametrize(
    ["capability_name", "response_type", "is_raised"],
    [("does_not_exist", HTTPPreconditionFailed, False), ("error", HTTPExpectationFailed, True)],
)
def test_enable_capability_error(
    test_config: Configurator,
    request_null_capability: DummyRequest,
    capability_name: str,
    response_type: HTTPException,
    is_raised: bool,
):
    """
    Tests that enable_capability view properly responds with HTTP exceptions given bad input

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability import enable_capability

    request_null_capability.matchdict["capability_name"] = capability_name
    if is_raised:
        with pytest.raises(response_type):
            response_enable_failed = enable_capability(request_null_capability)
            assert response_enable_failed.status_code == response_type.code
            assert type(response_enable_failed) is response_type
    else:
        response_enable_failed = enable_capability(request_null_capability)
        assert response_enable_failed.status_code == response_type.code
        assert type(response_enable_failed) is response_type


def test_disable_capability(test_config: Configurator, request_null_capability: DummyRequest):
    """
    Tests that capabilities can be properly disabled

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability import disable_capability

    request_null_capability.matchdict["capability_name"] = "null"
    response = disable_capability(request_null_capability)
    assert response.status_code == http.HTTPStatus.OK


@pytest.mark.parametrize(
    ["capability_name", "response_type", "is_raised"],
    [("does_not_exist", HTTPPreconditionFailed, False), ("error", HTTPExpectationFailed, True)],
)
def test_disable_capability_error(
    test_config: Configurator,
    request_null_capability: DummyRequest,
    capability_name: str,
    response_type: HTTPException,
    is_raised: bool,
):
    """
    Tests that disable_capability view properly responds with HTTP exceptions given bad input

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability import disable_capability

    request_null_capability.matchdict["capability_name"] = capability_name
    if is_raised:
        with pytest.raises(response_type):
            response_enable_failed = disable_capability(request_null_capability)
            assert response_enable_failed.status_code == response_type.code
            assert type(response_enable_failed) is response_type
    else:
        response_enable_failed = disable_capability(request_null_capability)
        assert response_enable_failed.status_code == response_type.code
        assert type(response_enable_failed) is response_type
