#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
from datetime import datetime
from unittest.mock import patch, MagicMock

from .test_workflow_info import FakeWorkflowInfo
from workspaces.system.schema import AbstractFile
from workspaces.workflow.services.remote_processing_service import CapoInjector
from workspaces.workflow.schema import Workflow, WorkflowRequest


"""
tests for remote_processing_service.CapoInjector
"""

expected_view = [
    "edu.nrao.workspaces.WorkflowSettings.serviceUrl",
    "edu.nrao.workspaces.NotificationSettings.serviceUrl",
    "edu.nrao.workspaces.UISettings.serviceUrl",
    "edu.nrao.workspaces.ProcessingSettings.useCasa",
    "edu.nrao.workspaces.ProcessingSettings.CasaVersion.vlass",
    "edu.nrao.archive.workflow.config.CasaVersions.homeForReprocessing",
]
expected_subspace = [
    "edu.nrao.workspaces.WorkflowSettings.serviceUrl = http://workflow:3456",
    "edu.nrao.workspaces.NotificationSettings.serviceUrl = http://notification:3458",
    "edu.nrao.workspaces.UISettings.serviceUrl = http://localhost:4444/workspaces",
    "edu.nrao.workspaces.ProcessingSettings.useCasa = false",
    "edu.nrao.workspaces.ProcessingSettings.CasaVersion.vlass = /home/casa/packages/pipeline/casa-6.1.3-3-pipeline-2021.1.1.32",
    "edu.nrao.archive.workflow.config.CasaVersions.homeForReprocessing = /home/casa/packages/pipeline/current",
]

request = WorkflowRequest(
    workflow_request_id=-3,
    workflow=Workflow(workflow_name="vlass_seci", uses_casa=True, remote_capable=True),
    workflow_name="vlass_seci",
    argument={"remote": True},
    state="Created",
    results_dir="/i/am/a/path",
    created_at=datetime(2021, 1, 1),
    updated_at=datetime(2021, 1, 1),
)


info = FakeWorkflowInfo()

injector = CapoInjector(info, request)


class TestCapoInjector:
    def test_is_remote_workflow(self):
        check = injector.is_remote_workflow()
        assert check is True

    def test_identify_subspace(self):
        view = injector.identify_subspace()
        assert view == expected_view

    def test_create_subspace(self):
        subspace = injector.create_subspace()
        assert subspace == expected_subspace

    def test_write_subspace(self):
        abs_file = injector.write_subspace()
        assert isinstance(abs_file, AbstractFile)
        assert abs_file.filename == "docker.properties"
        assert (
            abs_file.content
            == """edu.nrao.workspaces.WorkflowSettings.serviceUrl = http://workflow:3456\nedu.nrao.workspaces.NotificationSettings.serviceUrl = http://notification:3458\nedu.nrao.workspaces.UISettings.serviceUrl = http://localhost:4444/workspaces\nedu.nrao.workspaces.ProcessingSettings.useCasa = false\nedu.nrao.workspaces.ProcessingSettings.CasaVersion.vlass = /home/casa/packages/pipeline/casa-6.1.3-3-pipeline-2021.1.1.32\nedu.nrao.archive.workflow.config.CasaVersions.homeForReprocessing = /home/casa/packages/pipeline/current""".encode()
        )

    @patch("pathlib.Path.write_bytes")
    def test_subspace_to_directory(self, mock_write):
        injector.subspace_to_directory()
        assert mock_write.call_count == 1

    @patch("pathlib.Path.unlink")
    @patch("glob.glob")
    @patch("os.listdir")
    @patch("pathlib.Path.exists", MagicMock(return_value=True))
    def test_clear_subspace(self, mock_os, mock_glob, mock_remove):
        injector.clear_subspace()
        assert mock_os.call_count == 1
        assert mock_glob.call_count == 1
